﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Windows.Automation;
using System.Diagnostics;
using System.IO;
using FNF.Utility;
using FNF.Controls;
using FNF.XmlSerializerSetting;
using FNF.BouyomiChanApp;

namespace Plugin_wlm
{
    public partial class TargetForm : Form
    {
        // 監視ターゲット候補
        private class FoundElementItem
        {
            private string name;
            private AutomationElement elem;

            public string lasttoken;

            public string Name { get { return name; } }
            public AutomationElement Element { get { return elem; } }

            public FoundElementItem(string n, AutomationElement e)
            {
                name = n;
                elem = e;
            }

            public override string ToString()
            {
                return name;
            }
        }

        private Plugin_wlm plugin;

        private const int MAX_TALK_LENGTH = 200;
        private const string LEADING_STR = "箇条書き";
        private const string SEPARATOR_STR = "セパレーター";

        private List<FoundElementItem> activeTargets;

        public TargetForm()
        {
            InitializeComponent();
            activeTargets = new List<FoundElementItem>();
        }

        public Plugin_wlm Plugin{
            get { return plugin; }
            set { plugin = value; }
        }

        public void beginPlugin()
        {
            findIMWindow();
            clbTarget.Focus();
        }

        public void endPlugin()
        {
            activeTargets.Clear();
            timer1.Stop();
        }

        private void btnFindChat_Click(object sender, EventArgs e)
        {
            findIMWindow();
        }

        private void findIMWindow()
        {
            Cursor curorg = this.Cursor;
            this.Cursor = Cursors.WaitCursor;

            List<string> activenames = new List<string>(activeTargets.Count);
            foreach (FoundElementItem item in activeTargets)
            {
                activenames.Add(item.Name);
            }
            activeTargets.Clear();

            clbTarget.BeginUpdate();
            clbTarget.Items.Clear();

            AutomationElement ae = AutomationElement.RootElement;

            Condition cond = new OrCondition(
                new PropertyCondition(AutomationElement.ClassNameProperty, "IMWindowClass"),
                new PropertyCondition(AutomationElement.ClassNameProperty, "TabbedHostWndClass"));
            
            AutomationElementCollection aec = ae.FindAll(TreeScope.Children, cond);
            foreach (AutomationElement el in aec)
            {
                PropertyCondition pc1 = new PropertyCondition(AutomationElement.ControlTypeProperty, ControlType.Text);
                PropertyCondition pc2 = new PropertyCondition(AutomationElement.NameProperty, "履歴");

                AutomationElement dst = el.FindFirst(TreeScope.Descendants, new AndCondition(pc1, pc2));

                int i = clbTarget.Items.Add(new FoundElementItem(el.Current.Name, dst));

                if (activenames.IndexOf(el.Current.Name) != -1)
                {
                    clbTarget.SetItemChecked(i, true);
                }
            }

            clbTarget.EndUpdate();

            this.Cursor = curorg;
        }

        private void clbTarget_ItemCheck(object sender, ItemCheckEventArgs e)
        {
            if (e.CurrentValue == CheckState.Unchecked && e.NewValue == CheckState.Checked)
            {
                beginWatching((FoundElementItem)clbTarget.Items[e.Index]);
            }
            else if(e.CurrentValue == CheckState.Checked && e.NewValue == CheckState.Unchecked)
            {
                endWatching((FoundElementItem)clbTarget.Items[e.Index]);
            }
        }

        private void beginWatching(FoundElementItem target)
        {
            if (activeTargets.IndexOf(target) != -1) return;

            target.lasttoken = getLastToken(getStringOf(target.Element));

            activeTargets.Add(target);
            
            timer1.Start();
        }

        private void endWatching(FoundElementItem target)
        {
            if (activeTargets.IndexOf(target) == -1) return;

            activeTargets.Remove(target);

            if (activeTargets.Count == 0) timer1.Stop();
        }

        private string getLastToken(string history)
        {
            string trimhist = history.Trim();
            int lastblank = trimhist.LastIndexOf(' ');
            if (lastblank >= 0)
            {
                return trimhist.Substring(lastblank + 1);
            }
            else
            {
                return trimhist;
            }
        }

        private string getStringOf(AutomationElement e)
        {
            object obj;
            if (e.TryGetCurrentPattern(ValuePattern.Pattern, out obj))
            {
                ValuePattern vp = (ValuePattern)obj;
                return vp.Current.Value;
            }
            else
            {
                return "";
            }
        }

        private void timer1_Tick(object sender, EventArgs e)
        {
            foreach (FoundElementItem item in activeTargets)
            {
                getUpdate(item);
            }
        }

        private void getUpdate(FoundElementItem target)
        {
            string str = "";
            try
            {
                str = getStringOf(target.Element);
            }
            catch (Exception)
            {
                return;
            }

            if (str.Length == 0) return;
            if (getLastToken(str) == target.lasttoken) return;

            int lastindex = str.LastIndexOf(target.lasttoken);
            if (lastindex < 0 ||
                lastindex + target.lasttoken.Length < str.Length - MAX_TALK_LENGTH)
            {
                target.lasttoken = getLastToken(str);
                return;
            }

            string diffstr = str.Substring(lastindex + target.lasttoken.Length).Trim();
            bool silent = false;

            int leadingindex = diffstr.IndexOf(LEADING_STR);
            if (leadingindex >= 0)
            {
                diffstr = diffstr.Substring(leadingindex + LEADING_STR.Length).Trim();
            }
            else
            {
                if (diffstr.Length > SEPARATOR_STR.Length &&
                    diffstr.Substring(0, SEPARATOR_STR.Length) == SEPARATOR_STR)
                {
                    if (chkIgnoreSystemMessage.Checked)
                    {
                        silent = true;
                    }
                    else
                    {
                        diffstr = diffstr.Replace(SEPARATOR_STR, "").Trim();
                    }
                }
            }
            target.lasttoken = getLastToken(str);

            if (!silent)
            {
                Pub.AddTalkTask(diffstr);
            }
        }

        private void TargetForm_Load(object sender, EventArgs e)
        {

        }

        private void TargetForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            e.Cancel = true;
        }
    }
}
